# This file is part of cpu_cores released under the MIT license.
# See the LICENSE file for more information.

import unittest
import os

from cpu_cores import CPUCoresCounter


class TestLinux(unittest.TestCase):

    def _dir(self):
        return os.path.dirname(__file__)

    def test_factory_linux(self):
        x = CPUCoresCounter.factory(force_platform='linux')
        self.assertTrue(x is not None)
        self.assertTrue(x.platform is not None)
        self.assertTrue(x.platform == 'linux')

    def _test_linux_count(self, file_name, processors_count, cores_count):
        x = CPUCoresCounter.factory(force_platform='linux')
        self.assertTrue(x is not None)
        self.assertTrue(x.platform is not None)
        self.assertTrue(x.platform == 'linux')
        x._count(os.path.join(self._dir(), file_name))
        self.assertEqual(x._physical_processors_count, processors_count)
        self.assertEqual(x._physical_cores_count, cores_count)
        return x

    def test_linux_count1(self):
        self._test_linux_count('linux1', 2, 4)

    def test_linux_count2(self):
        self._test_linux_count('linux2', 1, 1)

    def test_linux_count3(self):
        self._test_linux_count('linux3', 2, 8)

    def test_linux_count4(self):
        self._test_linux_count('linux4', 1, 1)

    def test_linux_count5(self):
        self._test_linux_count('linux5', 1, 1)

    def test_linux_count6(self):
        self._test_linux_count('linux6', 1, 4)

    def test_linux_count7(self):
        self._test_linux_count('linux7', 1, 2)

    def test_linux_count8(self):
        self._test_linux_count('linux8', 2, 2)

    def test_linux_count9(self):
        self._test_linux_count('linux9', 2, 4)

if __name__ == '__main__':
    unittest.main()



# This file is part of cpu_cores released under the MIT license.
# See the LICENSE file for more information.

version_info = (0, 1, '1')
__version__ = ".".join([str(x) for x in version_info])

from cpu_cores.common import CPUCoresCounter

__all__ = ['CPUCoresCounter']



# This file is part of cpu_cores released under the MIT license.
# See the LICENSE file for more information.

import unittest
import os

from cpu_cores import CPUCoresCounter


class TestCPUCores(unittest.TestCase):

    def _dir(self):
        return os.path.dirname(__file__)

    def test_factory_darwin(self):
        x = CPUCoresCounter.factory(force_platform='darwin')
        self.assertTrue(x is not None)
        self.assertTrue(x.platform is not None)
        self.assertTrue(x.platform == 'darwin')

    def _test_darwin_count(self):
        x = CPUCoresCounter.factory(force_platform='darwin')
        self.assertTrue(x is not None)
        self.assertTrue(x.platform is not None)
        self.assertTrue(x.platform == 'darwin')
        return x

    def test_darwin_count1(self):
        x = self._test_darwin_count()
        x._count('cat %s' % os.path.join(self._dir(), 'darwin1'))
        self.assertEqual(x._physical_cores_count, 2)
        self.assertEqual(x._physical_processors_count, 1)


if __name__ == '__main__':
    unittest.main()



# This file is part of cpu_cores released under the MIT license.
# See the LICENSE file for more information.

import unittest
from cpu_cores import CPUCoresCounter


class TestCommon(unittest.TestCase):

    def test_factory(self):
        x = CPUCoresCounter.factory()
        self.assertTrue(x is not None)
        self.assertTrue(x.platform is not None)
        self.assertTrue(len(x.platform) > 0)

    def _test_unkwown_platform(self):
        CPUCoresCounter.factory(force_platform='unknown')

    def _test_bad_call(self):
        x = CPUCoresCounter()
        x._count()

    def test_unknown_platform(self):
        self.assertRaises(NotImplementedError, self._test_unkwown_platform)

    def test_bad_call(self):
        self.assertRaises(NotImplementedError, self._test_bad_call)

    def _test_get_common(self):
        x = CPUCoresCounter.factory()
        self.assertTrue(x is not None)
        self.assertTrue(x.platform is not None)
        self.assertTrue(len(x.platform) > 0)
        return x

    def test_get_physical_cores_count(self):
        x = self._test_get_common()
        c = x.get_physical_cores_count()
        self.assertTrue(c > 0)

    def test_get_physical_processors_count(self):
        x = self._test_get_common()
        c = x.get_physical_processors_count()
        self.assertTrue(c > 0)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# This file is part of cpu_cores released under the MIT license.
# See the LICENSE file for more information.

import argparse
import sys

from cpu_cores import CPUCoresCounter

description = 'Get the number of real physical cpu cores and processors'
parser = argparse.ArgumentParser(description=description)
parser.add_argument("-s", "--single-value", type=str, default=None,
                    dest="single_value",
                    help="just display a single value "
                         "(use 'cores' or 'processors')")
args = parser.parse_args()

single_value = None
if args.single_value:
    if args.single_value not in ('cores', 'processors'):
        stderr = sys.stderr
        stderr.write("ERROR : single_value must be 'cores' or 'processors'\n")
        parser.print_help()
    single_value = args.single_value

try:
    c = CPUCoresCounter.factory()
    procs_count = c.get_physical_processors_count()
    cores_count = c.get_physical_cores_count()
except:
    sys.stderr.write("Can't read the number of processors/cores"
                     "(platform=%s)\n" % c.platform)
    sys.exit(1)

if single_value == "cores":
    print(cores_count)
elif single_value == "processors":
    print(procs_count)
else:
    print("Number of real physical processors : %i" % procs_count)
    print("Number of real physical cores      : %i" % cores_count)



# This file is part of cpu_cores released under the MIT license.
# See the LICENSE file for more information.

import shlex
import subprocess

from cpu_cores.common import CPUCoresCounter

CPUINFO_COMMAND = "/usr/sbin/system_profiler" \
                  " -detailLevel full SPHardwareDataType"


class DarwinCPUCoresCounter(CPUCoresCounter):

    def _count(self, command=None):
        if command is None:
            command = CPUINFO_COMMAND
        s = subprocess.Popen(shlex.split(command),
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        if s:
            out, err = s.communicate()
            if len(err.strip()) > 0 or len(out.strip()) == 0:
                raise Exception('impossible to get the cpu cores count' +
                                '(darwin) (error message = %s)' % err.strip())
            lines = out.split(b'\n')
            for line in lines:
                tmp = line.strip()
                if tmp.startswith(b'Total Number of Cores:'):
                    self._physical_cores_count = int(tmp.split(b':')[1])
                if tmp.startswith(b'Number of Processors:'):
                    self._physical_processors_count = int(tmp.split(b':')[1])
        if self._physical_processors_count is None or \
                self._physical_cores_count is None:
            raise Exception('impossible to get the cpu cores count (darwin)')



# This file is part of cpu_cores released under the MIT license.
# See the LICENSE file for more information.

from cpu_cores.common import CPUCoresCounter

CPUINFO_FILEPATH = "/proc/cpuinfo"


def _core_hash(cpu_infos):
    if 'core id' not in cpu_infos and 'physical id' not in cpu_infos:
        return "%i" % cpu_infos['processor']
    if 'core id' in cpu_infos and 'physical id' not in cpu_infos:
        raise Exception("incorrect cpuinfo file :"
                        " we have a core_id without physical_id")
    if 'core id' in cpu_infos:
        return "%i_%i" % (cpu_infos['physical id'], cpu_infos['core id'])
    else:
        return "%i" % cpu_infos['physical id']


def _processor_hash(cpu_infos):
    if 'core id' not in cpu_infos and 'physical id' not in cpu_infos:
        return "%i" % cpu_infos['processor']
    if 'core id' in cpu_infos and 'physical id' not in cpu_infos:
        raise Exception("incorrect cpuinfo file :"
                        " we have a core_id without physical_id")
    return "%i" % cpu_infos['physical id']


class LinuxCPUCoresCounter(CPUCoresCounter):

    def _count(self, cpuinfo_filepath=None):
        if cpuinfo_filepath is None:
            cpuinfo_filepath = CPUINFO_FILEPATH
        with open(cpuinfo_filepath, 'r') as f:
            # we read lines in reversed order to be sure to end with a
            # "processor:" line
            lines = reversed(f.readlines())
            cores = set()
            processors = set()
            cpu_infos = {}
            for line in lines:
                tmp = line.strip()
                for key in ('processor', 'physical id', 'core id'):
                    if tmp.startswith(key):
                        cpu_infos[key] = int(tmp.split(':')[1].strip())
                        if key == 'processor':
                            cores.add(_core_hash(cpu_infos))
                            processors.add(_processor_hash(cpu_infos))
                            cpu_infos = {}
            if len(cores) == 0 or len(processors) == 0:
                raise Exception("can't get the cpu cores count (linux)")
        self._physical_cores_count = len(cores)
        self._physical_processors_count = len(processors)



# This file is part of cpu_cores released under the MIT license.
# See the LICENSE file for more information.

import sys


class CPUCoresCounter(object):

    platform = None
    _physical_cores_count = None
    _physical_processors_count = None

    def _count(self, *args, **kwargs):
        raise NotImplementedError()

    @classmethod
    def factory(cls, force_platform=None):
        if force_platform is not None:
            cls.platform = force_platform
        else:
            cls.platform = sys.platform
        if cls.platform.startswith('darwin'):
            from cpu_cores.darwin import DarwinCPUCoresCounter
            return DarwinCPUCoresCounter()
        elif cls.platform.startswith('linux'):
            from cpu_cores.linux import LinuxCPUCoresCounter
            return LinuxCPUCoresCounter()
        else:
            raise NotImplementedError("unsupported platform type [%s]" %
                                      cls.platform)

    def _check_counting_or_do_it(self):
        if self._physical_processors_count is None or \
                self._physical_cores_count is None:
            self._count()

    def get_physical_cores_count(self):
        self._check_counting_or_do_it()
        return self._physical_cores_count

    def get_physical_processors_count(self):
        self._check_counting_or_do_it()
        return self._physical_processors_count
